@@ -14,7 +14,9 @@ has fields => (
 );
 
 sub _validate_fields {
-    my $self = shift;
+    my ( $self, $args ) = @_;
+
+    return unless defined $args->{fields};
     return unless $self->has_fields;
 
     my $fields = $self->fields;
@@ -73,7 +75,7 @@ PONAPI::DAO::Request::Role::HasFields - DAO request role - `fields`
 
 =head1 VERSION
 
-version 0.002013
+version 0.002014
 
 =head1 AUTHORS
 
