@@ -14,7 +14,9 @@ has filter => (
 );
 
 sub _validate_filter {
-    my $self = shift;
+    my ( $self, $args ) = @_;
+
+    return unless defined $args->{filter};
 
     $self->has_filter
         or $self->_bad_request( "`filter` is missing values" );
@@ -36,7 +38,7 @@ PONAPI::DAO::Request::Role::HasFilter - DAO request role - `filter`
 
 =head1 VERSION
 
-version 0.002013
+version 0.002014
 
 =head1 AUTHORS
 
