@@ -9,6 +9,13 @@ has id => (
     predicate => 'has_id',
 );
 
+sub _validate_id {
+    my $self = shift;
+
+    $self->_bad_request( "`id` is missing for this request" )
+        unless $self->has_id;
+}
+
 no Moose::Role; 1;
 
 __END__
@@ -23,7 +30,7 @@ PONAPI::DAO::Request::Role::HasID - DAO request role - `id`
 
 =head1 VERSION
 
-version 0.002013
+version 0.002014
 
 =head1 AUTHORS
 
