@@ -14,12 +14,15 @@ has include => (
 );
 
 sub _validate_include {
-    my $self = shift;
-    my $type = $self->type;
+    my ( $self, $args ) = @_;
+
+    return unless defined $args->{include};
 
     return $self->_bad_request( "`include` is missing values" )
         unless $self->has_include >= 1;
 
+    my $type = $self->type;
+
     for ( @{ $self->include } ) {
         $self->repository->has_relationship( $type, $_ )
             or $self->_bad_request( "Types `$type` and `$_` are not related", 404 );
@@ -40,7 +43,7 @@ PONAPI::DAO::Request::Role::HasInclude - DAO request role - `include`
 
 =head1 VERSION
 
-version 0.002013
+version 0.002014
 
 =head1 AUTHORS
 
