@@ -13,7 +13,9 @@ has page => (
 );
 
 sub _validate_page {
-    my $self = shift;
+    my ( $self, $args ) = @_;
+
+    return unless defined $args->{page};
 
     $self->has_page
         or $self->_bad_request( "`page` is missing values" );
@@ -35,7 +37,7 @@ PONAPI::DAO::Request::Role::HasPage - DAO request role - `page`
 
 =head1 VERSION
 
-version 0.002013
+version 0.002014
 
 =head1 AUTHORS
 
