@@ -10,9 +10,9 @@ has rel_type => (
 );
 
 sub _validate_rel_type {
-    my $self = shift;
+    my ( $self, $args ) = @_;
 
-    return $self->_bad_request( "`relationship type` is missing" )
+    return $self->_bad_request( "`relationship type` is missing for this request" )
         unless $self->has_rel_type;
 
     my $type     = $self->type;
@@ -37,7 +37,7 @@ PONAPI::DAO::Request::Role::HasRelationshipType - DAO request role - `relationsh
 
 =head1 VERSION
 
-version 0.002013
+version 0.002014
 
 =head1 AUTHORS
 
