@@ -14,7 +14,9 @@ has sort => (
 );
 
 sub _validate_sort {
-    my $self = shift;
+    my ( $self, $args ) = @_;
+
+    return unless defined $args->{sort};
 
     $self->has_sort
         or $self->_bad_request( "`sort` is missing values" );
@@ -36,7 +38,7 @@ PONAPI::DAO::Request::Role::HasSort - DAO request role - `sort`
 
 =head1 VERSION
 
-version 0.002013
+version 0.002014
 
 =head1 AUTHORS
 
